#include "info_module.h"
#include "bsp_ethernet_driver.h"
#include "Mikromedia_7_Demo_objects.h"

/* ------------------------------------------------------------------- MACROS */



/* ---------------------------------------------------------------- VARIABLES */

static module_t         info_module;

#if ( _MMB_MODULE_ETHERNET == 1 )

#include "test_server_page.h"

static const uint8_t mac_address[6] = { 0x00, 0x14, 0xA5, 0x76, 0x19, 0x3F };

/**
    Device parameters :

        - IP Address
        - Gateway Address
        - Network Mask
        - DNS Address
*/
        
static const uint8_t ip_address[4]  = {  10, 101,  14,  39 };
static const uint8_t gw_address[4]  = {  10, 101,  14,   1 };
static const uint8_t net_mask[4]    = { 255, 255, 255,   0 };
static const uint8_t dns_address[4] = {  10, 101,  14,  99 };

//void print_ip ( uint8_t * hostname, uint8_t * ip )
//{
//    test_console_output( hostname, ( void * ) &ip[0], BYTE_FORMAT, "." );
//    test_console_output( "", ( void * ) &ip[1], BYTE_FORMAT, "." );
//    test_console_output( "", ( void * ) &ip[2], BYTE_FORMAT, "." );
//    test_console_output( "", ( void * ) &ip[3], BYTE_FORMAT, "\r\n" );
//}

uint16_t Ethernet_Intern_UserTCP ( uint8_t * remoteHost, uint16_t remotePort, uint16_t localPort, uint16_t reqLength, TEthInternPktFlags * flags )
{
    uint8_t     buf[32];
    uint16_t    len;        //  Reply length.

    /**
        Listen only to web request on port 80.
    */

    if ( localPort != 80 )
    {
        return 0;
    }

    /**
        Get 10 first bytes only of the request, the rest does not matter here.
    */

    for ( len = 0; len < 10; len++ )
    {
        buf[len] = Ethernet_Intern_getByte( );
    }

    buf[len] = 0;
    len = 0;

    /**  
        Only GET method supported.
    */

    if ( memcmp( buf, HTTP_GET_METHOD, 5 ) )
    {
        return 0;
    }

//    print_ip( "HTTP request from client", remoteHost );

    if ( len == 0 )
    {
        len =  Ethernet_Intern_putConstString( HTTP_HEADER );
        len += Ethernet_Intern_putConstString( HTTP_MIME_HTML_TYPE );
        len += Ethernet_Intern_putConstString( HTTP_WEB_PAGE );
    }

    return len;
}

uint16_t Ethernet_Intern_UserUDP ( uint8_t * remoteHost, uint16_t remotePort, uint16_t destPort, uint16_t reqLength, TEthInternPktFlags * flags )
{
    uint8_t  buf[32];
    uint16_t len;

    if ( destPort != 10001 )
    {
        return 0;
    }

    /**
        Reply is made of the remote host IP address in human readable format.
    */

    memset( buf, 0 , 32 );

    ByteToStr( remoteHost[0], buf );        buf[3] = '.';
    ByteToStr( remoteHost[1], buf + 4 );    buf[7] = '.';
    ByteToStr( remoteHost[2], buf + 8 );    buf[11] = '.';
    ByteToStr( remoteHost[3], buf + 12 );

    buf[15] = ':';

    /**
        Then remote host port number.
    */

    WordToStr( remotePort, buf + 16 );      buf[21] = '[';
    WordToStr( destPort, buf + 22 );        buf[27] = ']';

    /**
        The total length of the request is the length of 
        the dynamic string plus the text of the request.
    */

    len = 28 + reqLength;

    /**
        Puts the dynamic string into the transmit buffer.
    */

    Ethernet_Intern_putBytes( buf, 28 );

    /**
        Then puts the request string converted into
        upper char into the transmit buffer.
    */

    while ( reqLength-- )
    {
        char tmp = Ethernet_Intern_getByte( );
        Ethernet_Intern_putByte( toupper( tmp ) );
    }

    return len;
}

#endif

/* -------------------------------------------- PRIVATE FUNCTION DECLARATIONS */



/* ----------------------------------------------------------- IMPLEMENTATION */

void info_module_init ( module_t ** module )
{
    info_module.state = MODULE_INACTIVE;
    info_module.menu = MENU_COLLAPSED;
    
//    #if (  _MMB_MODULE_ETHERNET == 1 )
//
//        Ethernet_Intern_SoftwareReset( );
//        Delay_ms(100);
//        bsp_ethernet_init( mac_address, ip_address );
//        bsp_ethernet_network_init( gw_address, net_mask, dns_address );
//
//    #endif
    
    *module = &info_module;
}

void info_module_task ( void )
{
    switch ( info_module.state )
    {
        case MODULE_INACTIVE :
        {
            DrawScreen( &Screen_05_Info );

            info_module.state = MODULE_ACTIVE;

//            #if (  _MMB_MODULE_ETHERNET == 1 )
//
//                Ethernet_Intern_SoftwareReset( );
//                Delay_ms(100);
//                bsp_ethernet_init( mac_address, ip_address );
//                bsp_ethernet_network_init( gw_address, net_mask, dns_address );
//
//            #endif
            
            break;
        }
        case MODULE_ACTIVE :
        {
            switch ( info_module.menu )
            {
                case MENU_COLLAPSED :
                {
//                    #if ( _MMB_MODULE_ETHERNET == 1 )
//
//                        Ethernet_Intern_doPacket( );
//
//                    #endif

                    break;
                }
                case MENU_EXPANDED :
                {
//                    Ethernet_Intern_doPacket( );

                    break;
                }
            }

            break;
        }
        default :
        {
            /** ERROR */
                
            break;
        }
    }
}