#include "mp3_module.h"
#include "bsp_mp3_driver.h"
#include "bsp_microsd_driver.h"
#include "Mikromedia_7_Demo_objects.h"

/* ------------------------------------------------------------------- MACROS */



/* ---------------------------------------------------------------- VARIABLES */

static module_t         mp3_module;

#if (  _MMB_MODULE_MP3 == 1 )

    static mp3_player_state_t   mp3_state;
    static uint8_t              mp3_volume;
    static mp3_playlist_t       mp3_playlist[3];
    static mp3_song_number_t    mp3_song_number;

#endif

#if (  _MMB_MODULE_MICROSD == 1 )

    static int8_t      sd_err;
    static int8_t      sd_fd;
    static uint32_t    sd_seek;
    static sd_file_t   sd_file;

#endif

/* -------------------------------------------- PRIVATE FUNCTION DECLARATIONS */

/**

*/
static void player_running ( void );

/**

*/
static void player_stopped ( void );

/* ----------------------------------------------------------- IMPLEMENTATION */

void increase_volume( void )
{
    uint16_t aux_volume;

    if (mp3_volume > 0x00)
    {
        mp3_volume--;

        aux_volume = 0x00;
        aux_volume |= mp3_volume;
        aux_volume <<= 8;
        aux_volume |= mp3_volume;

        bsp_mp3_cmd_write( _BSP_MP3_VOL_ADDR, aux_volume );
    }
}

void decrease_volume( void )
{
    uint16_t aux_volume;

    if (mp3_volume < 0xFD)
    {
        mp3_volume++;

        aux_volume = 0x00;
        aux_volume |= mp3_volume;
        aux_volume <<= 8;
        aux_volume |= mp3_volume;

        bsp_mp3_cmd_write( _BSP_MP3_VOL_ADDR, aux_volume );
    }
}

void mute_volume( void )
{
    static uint8_t aux_flag = 0;
    uint16_t aux_volume;

    switch (aux_flag)
    {
        case 0 :
        {
            bsp_mp3_cmd_write( _BSP_MP3_VOL_ADDR, 0xFDFD );
            
            aux_flag = 1;
            
            break;
        }
        case 1 :
        {
            aux_volume = 0x00;
            aux_volume |= mp3_volume;
            aux_volume <<= 8;
            aux_volume |= mp3_volume;

            bsp_mp3_cmd_write( _BSP_MP3_VOL_ADDR, aux_volume );

            aux_flag = 0;
            
            break;
        }
        default :
        {
            break;
        }
    }
}

void mp3_next_song( void )
{
    player_stopped( );
    
    switch (mp3_song_number)
    {
        case SONG_1:
        {
            mp3_song_number = SONG_2;

            break;
        }
        case SONG_2:
        {
            mp3_song_number = SONG_3;

            break;
        }
        case SONG_3:
        {
            mp3_song_number = SONG_1;
        
            break;
        }
        default :
        {
            break;
        }
    }
    
    sd_file.sd_file_size = 0;
    sd_seek = 0;
}

void mp3_prev_song( void )
{
    player_stopped( );

    switch (mp3_song_number)
    {
        case SONG_1:
        {
            mp3_song_number = SONG_3;

            break;
        }
        case SONG_2:
        {
            mp3_song_number = SONG_1;

            break;
        }
        case SONG_3:
        {
            mp3_song_number = SONG_2;

            break;
        }
        default :
        {
            break;
        }
    }

    sd_file.sd_file_size = 0;
    sd_seek = 0;
}

void mp3_module_state_changed ( void )
{
    switch (mp3_state)
    {
        case MP3_PLAY:
        {
            mp3_state = MP3_STOP;
            break;
        }
        case MP3_STOP:
        {
            mp3_state = MP3_PLAY;
            break;
        }
        default :
        {
            break;
        }
    }
}

/*
    Called from system initialization procedure.
*/

void mp3_module_init ( module_t ** module )
{
    mp3_module.state = MODULE_INACTIVE;
    mp3_module.menu = MENU_COLLAPSED;
    
    #if ( _MMB_MODULE_MP3 == 1 )

        bsp_mp3_init( );
        
        bsp_mp3_cmd_write( _BSP_MP3_MODE_ADDR,   0x0800 );
        bsp_mp3_cmd_write( _BSP_MP3_BASS_ADDR,   0x7A00 );
        bsp_mp3_cmd_write( _BSP_MP3_CLOCKF_ADDR, 0xC000 );
        Delay_ms(500);
        bsp_mp3_cmd_write( _BSP_MP3_VOL_ADDR,    0x0000 );

        mp3_volume      = 0x00;
        mp3_song_number = SONG_1;
        
        strncpy( mp3_playlist[ SONG_1 ].SONG, "song_1.mp3", 11 );
        strncpy( mp3_playlist[ SONG_2 ].SONG, "song_2.mp3", 11 );
        strncpy( mp3_playlist[ SONG_3 ].SONG, "song_3.mp3", 11 );

    #endif

    #if ( _MMB_MODULE_MICROSD == 1 )

        bsp_microsd_init( );

        if ( ( sd_err = FAT32_Init( ) ) < 0 )
        {
            /** ERROR */
        }

        /**
            Navigate to the root directory.
        */

        if ( ( sd_err = FAT32_ChangeDir( "\\" ) ) < 0 )
        {
            /** ERROR */
        }
        
        sd_file.sd_file_size = 0;
        sd_file.sd_file_state = SD_FILE_CLOSED;

    #endif

    *module = &mp3_module;
}

void mp3_module_task ( void )
{
    switch ( mp3_module.state )
    {
        case MODULE_INACTIVE :
        {
            DrawScreen( &Screen_03_MP3 );

            mp3_module.state = MODULE_ACTIVE;
            mp3_state = MP3_STOP;

            break;
        }
        case MODULE_ACTIVE :
        {
            switch ( mp3_module.menu )
            {
                case MENU_COLLAPSED :
                {
                    switch ( mp3_state )
                    {
                        case MP3_STOP :
                        {
                            player_stopped( );
                            
                            break;
                        }
                        case MP3_PLAY :
                        {
                            player_running( );
                            
                            break;
                        }
                        default :
                        {
                            /** ERROR */

                            break;
                        }
                    }

                    break;
                }
                case MENU_EXPANDED :
                {


                    break;
                }
            }

            break;
        }
        default :
        {
            /** ERROR */

            break;
        }
    }
}

/* --------------------------------------------- PRIVATE FUNCTION DEFINITIONS */

static void player_running ( void )
{
    static uint8_t  i;

    char aux_txt[50];

    switch (sd_file.sd_file_state)
    {
        case SD_FILE_CLOSED:
        {
            sd_fd = 0;

            if (( sd_fd = FAT32_Open( mp3_playlist[ mp3_song_number ].SONG, FILE_READ ) ) >= 0)
            {
                /** SUCCESS */
            }
            else
            {
                while (1); /** ERROR */
            }

            if (( sd_err = FAT32_Size( mp3_playlist[ mp3_song_number ].SONG, &sd_file.sd_file_size ) ) >= 0)
            {
                sd_seek = 0; /** SUCCESS */
            }
            else
            {
                while (1); /** ERROR */
            }
            
            sd_file.sd_file_state = SD_FILE_OPEN;
            
            break;
        }
        case SD_FILE_OPEN:
        {
            if (sd_file.sd_file_size > 32)
            {
                if (( sd_err = FAT32_Read( sd_fd, sd_file.sd_file_data, 32 ) ) >= 0)
                {
                    for (i = 0; i < 32; i ++)
                    {
                        while ( bsp_mp3_data_write( sd_file.sd_file_data[ i ] ) );
                    }
                    
                    sd_file.sd_file_size -= 32;
                    sd_seek += 32;
                    
                    FAT32_Seek( sd_fd, sd_seek );
                }
                else
                {
                    while (1); /** ERROR */
                }
            }
            else if (sd_file.sd_file_size > 0)
            {
                if (( sd_err = FAT32_Read( sd_fd, sd_file.sd_file_data, sd_file.sd_file_size ) ) >= 0)
                {
                    for (i = 0; i < sd_file.sd_file_size; i ++)
                    {
                        while ( bsp_mp3_data_write( sd_file.sd_file_data[ i ] ) );
                    }
                    
                    sd_file.sd_file_size = 0;
                    sd_seek = 0;
                }
                else
                {
                    while (1); /** ERROR */
                }
            }
            else
            {
                switch (sd_file.sd_file_state)
                {
                    case SD_FILE_CLOSED:
                    {
                        while (1); /** ERROR */
                        
                        break;
                    }
                    case SD_FILE_OPEN:
                    {
                        if (( sd_err = FAT32_Close( sd_fd ) ) >= 0)
                        {
                            /** SUCCESS */
                        }
                        else
                        {
                            while (1); /** ERROR */
                        }

                        sd_file.sd_file_state = SD_FILE_CLOSED;
                        
                        break;
                    }
                    default :
                    {
                        break;
                    }
                }
            }

            break;
        }
        default :
        {
            break;
        }
    }
}

static void player_stopped ( void )
{
    switch (sd_file.sd_file_state)
    {
        case SD_FILE_CLOSED:
        {
            /** NOT AN ERROR */

            break;
        }
        case SD_FILE_OPEN:
        {
            if (( sd_err = FAT32_Close( sd_fd ) ) >= 0)
            {
                sd_file.sd_file_state = SD_FILE_CLOSED;
                
                sd_file.sd_file_size = 0;
                sd_seek = 0;
            }
            else
            {
                while (1); /** ERROR */
            }

            break;
        }
        default :
        {
            break;
        }
    }
}