#include "remote_module.h"
#include "bsp_ir_driver.h"
#include "bsp_rf_driver.h"
#include "bsp_rgb_driver.h"
#include "bsp_buzzer_driver.h"
#include "Mikromedia_7_Demo_objects.h"

/* ------------------------------------------------------------------- MACROS */



/* ---------------------------------------------------------------- VARIABLES */

static module_t         remote_module;

#if (  _MMB_MODULE_IR == 1 )

    static uint8_t  ir_data[3];
    static uint8_t  ir_error;

#endif

#if (  _MMB_MODULE_RF == 1 )

    static uint8_t TX_ADDRESS[5] = { 0xE7, 0xE7, 0xE7, 0xE7, 0xE7 };
    static uint8_t RX_ADDRESS[5] = { 0xD7, 0xD7, 0xD7, 0xD7, 0xD7 };

    static uint8_t rx_buffer[32];
  
#endif

#if (  _MMB_MODULE_BUZZER == 1 )

    static const uint16_t BUZZER_SCALES[3][12] =
    {
        {  523,  554,  587,  622,  659,  698,  740,  784,  831,  880,  932,  988 },
        { 1046, 1108, 1174, 1244, 1318, 1397, 1480, 1568, 1662, 1760, 1865, 1975 },
        { 2093, 2217, 2349, 2489, 2637, 2794, 2960, 3136, 3322, 3520, 3729, 3951 }
    };
    
    static uint8_t aux_ctr;

#endif

/*
    Called from system initialization procedure.
*/

/* -------------------------------------------- PRIVATE FUNCTION DECLARATIONS */



/* ----------------------------------------------------------- IMPLEMENTATION */

//static void update_label( char * text )
//{
//    LABEL_MP3.Font_Color = CL_BLACK;
//    DrawLabel( &LABEL_MP3 );
//    LABEL_MP3.Font_Color = CL_GREEN;
//    LABEL_MP3.Caption = text;
//    DrawLabel( &LABEL_MP3 );
//}

void remote_module_init ( module_t ** module )
{
    remote_module.state = MODULE_INACTIVE;
    remote_module.menu  = MENU_COLLAPSED;

    *module = &remote_module;
}

void remote_module_task ( void )
{
    switch ( remote_module.state )
    {
        case MODULE_INACTIVE :
        {
            DrawScreen( &Screen_04_Remote );

            #if (  _MMB_MODULE_IR == 1 )

                bsp_ir_init( 1200 );
                
                aux_ctr = 0;

            #endif

            #if (  _MMB_MODULE_RF == 1 )

                /*  
                    Initialize BSP module.
                    
                        - Initialize driver
                        - Set channel 2
                        - Payload 8 bytes
                */
    
                /* Initialize hardware pins */
    
                bsp_rf_init( );
    
                /* Channel #2, payload length: 4 */
                /* Set the device addresses */
    
                bsp_rf_config( 2, 8 );
                bsp_rf_tx_address( TX_ADDRESS );
                bsp_rf_rx_address( RX_ADDRESS );

            #endif

            #if (  _MMB_MODULE_BUZZER == 1 )

                bsp_buzzer_init( );

            #endif
            
            remote_module.state = MODULE_ACTIVE;

            break;
        }
        case MODULE_ACTIVE :
        {
            switch ( remote_module.menu )
            {
                case MENU_COLLAPSED :
                {
                    #if ( ( _MMB_MODULE_IR == 1 ) && ( _MMB_MODULE_BUZZER == 1 ) )
                    
                        if (remote_module.menu == MENU_COLLAPSED)
                        {
                            if ( bsp_ir_pin( ) == 0 )
                            {
                                bsp_buzzer_play( BUZZER_SCALES[1][ aux_ctr ], 50 );
                            }

                            if (aux_ctr < 11)
                            {
                               aux_ctr++;
                            }
                            else
                            {
                                aux_ctr = 0;
                            }

                            Check_TP( );
                        }
                        
                    #endif

                    #if ( ( _MMB_MODULE_RF == 1 ) && ( _MMB_MODULE_RGBLED == 1 ) )
                    
                        if (remote_module.menu == MENU_COLLAPSED)
                        {
                            if ( bsp_rf_data_ready( ) != 0 )
                            {
                                bsp_rf_data_read( rx_buffer );
                            }
                        
                            Check_TP( );
                        }
                        
                    #endif

                    break;
                }
                case MENU_EXPANDED :
                {
//                    Ethernet_Intern_doPacket( );

                    break;
                }
            }

            break;
        }
        default :
        {
            /* ERROR */

            break;
        }
    }
}