#include "status_module.h"
#include "bsp_power_driver.h"
#include "bsp_accel_driver.h"
#include "bsp_rtc_driver.h"
#include "bsp_temp_driver.h"
#include "bsp_als_driver.h"
#include "bsp_wifi_driver.h"
#include "api_wifi.h"
#include "Mikromedia_7_Demo_objects.h"
#include "bsp.h"

/* ------------------------------------------------------------------- MACROS */

#define REFRESH_RATE 10
#define REFRESH_RATE_WIFI 10

/* ---------------------------------------------------------------- VARIABLES */

static module_t         status_module;

#if ( _MMB_MODULE_POWER == 1 )

    static float power;
    static int16_t x_axis;
    static int16_t y_axis;
    static int16_t z_axis;
  
#endif

#if ( _MMB_MODULE_RTC == 1 )

    static rtc_time_t rtc_time;
  
    static rtc_time_t MY_TIME =
    {
        0,     // am-pm

        0,     // hour
        0,     // minute
        0,     // second

        0,     // day
        0,     // month
        0      // year
    };
  
#endif

#if ( _MMB_MODULE_TEMP == 1 )

    static float temperature;
  
#endif

#if ( _MMB_MODULE_ALS == 1 )

    static uint8_t ALS_CTRL_REG = 0x01;
    static uint8_t ALS_MEAS_REG = 0x12;
    static uint8_t als_manufac_id;
    static uint8_t als_part_id;
    static uint16_t channel_0;
    static uint16_t channel_1;
  
#endif

#if ( _MMB_MODULE_WIFI == 1 )

    static int32_t retVal;
    static uint8_t i;

#endif

static char text[50];
static uint8_t aux_ctr;
static uint8_t aux_wifi_ctr;

/* -------------------------------------------- PRIVATE FUNCTION DECLARATIONS */

/**
    This function rounds floating point value
    converted to string to two decimal places
*/
static void float_cut( char * text );

/* ----------------------------------------------------------- IMPLEMENTATION */

void status_module_init ( module_t ** module )
{
    status_module.state = MODULE_INACTIVE;
    status_module.menu = MENU_COLLAPSED;

    aux_ctr = REFRESH_RATE;
    aux_wifi_ctr = REFRESH_RATE_WIFI;

    *module = &status_module;
}

void status_module_task ( void )
{
    switch ( status_module.state )
    {
        case MODULE_INACTIVE :
        {
            DrawScreen( &Screen_01_Status );
            
            #if ( _MMB_MODULE_POWER == 1 )

                bsp_power_init( );

            #endif

            #if (  _MMB_MODULE_ACCEL == 1 )

                bsp_accel_init( );

            #endif

            #if (  _MMB_MODULE_RTC == 1 )

                bsp_rtc_init( );
                bsp_rtc_set( &MY_TIME );

            #endif

            #if (  _MMB_MODULE_TEMP == 1 )

                bsp_temperature_init( );

            #endif

            #if (  _MMB_MODULE_ALS == 1 )

                bsp_als_init( );

                bsp_als_write_regs( _BSP_ALS_CONTR_REG,      &ALS_CTRL_REG,   1 );
                bsp_als_write_regs( _BSP_ALS_MEAS_REG,       &ALS_MEAS_REG,   1 );
                bsp_als_read_regs(  _BSP_ALS_MANUFAC_ID_REG, &als_manufac_id, 1 );
                bsp_als_read_regs(  _BSP_ALS_PART_ID_REG,    &als_part_id,    1 );

            #endif

            #if (  _MMB_MODULE_WIFI == 1 )

                bsp_wifi_init(  );
                SPI_Set_Active( SPI1_Read, SPI1_Write );
                
                retVal = initializeAppVariables( );
                api_wifi_prerror( retVal );

                retVal = configureSimpleLinkToDefaultState( );

                if ( retVal < 0 )
                {
                    if ( DEVICE_NOT_IN_STATION_MODE == retVal )
                    {
                        test_console_write_line( "Failed to configure the device" );
                    }

                    return;
                }

                test_console_write_line( "Device is configured in default state" );

                retVal = sl_Start( 0, 0, 0 );

                if ( ( retVal < 0 ) || ( ROLE_STA != retVal ) )
                {
                    test_console_write_line( "Failed to start the device" );

                    return;
                }

                test_console_write_line( "Device started as STATION" );

            #endif

            status_module.state = MODULE_ACTIVE;

            break;
        }
        case MODULE_ACTIVE :
        {
            switch ( status_module.menu )
            {
                case MENU_COLLAPSED :
                {
                    #if ( _MMB_MODULE_POWER == 1 )
                    
                        if (( status_module.menu == MENU_COLLAPSED ) && ( aux_ctr == REFRESH_RATE ))
                        {
                            power = bsp_power_read( );
                        
                            Label_01_00_Power_Caption[0] = 0;

                            text[0] = 0;
                            FloatToStr( power, Label_01_00_Power_Caption );
                            Ltrim( Label_01_00_Power_Caption );
                            float_cut( Label_01_00_Power_Caption );
                            Label_01_00_Power_Caption[ strlen( Label_01_00_Power_Caption ) ] = 0;
                            strcat( Label_01_00_Power_Caption, "   V" );

                            DrawImage( &Image_01_02_Power );
                            DrawLabel( &Label_01_00_Power );
                        
                            Check_TP( );
                        }
                        
                    #endif

                    #if ( _MMB_MODULE_ACCEL == 1 )
                    
                        if (status_module.menu == MENU_COLLAPSED)
                        {
                            bsp_accel_read_measurement( &x_axis, &y_axis, &z_axis );
                            
                            Label_01_01_Accel_Caption[0] = 0;

                            text[0] = 0;                            
                            strcat( Label_01_01_Accel_Caption, "X :   " );                            
                            IntToStr( x_axis, text );
                            Ltrim( text );
                            strcat( Label_01_01_Accel_Caption, text );
                            text[0] = 0;
                            strcat( Label_01_01_Accel_Caption, "   |   Y :   " );
                            IntToStr( y_axis, text );
                            Ltrim( text );
                            strcat( Label_01_01_Accel_Caption, text );
                            text[0] = 0;
                            strcat( Label_01_01_Accel_Caption, "   |   Z :   " );
                            IntToStr( z_axis, text );
                            Ltrim( text );
                            strcat( Label_01_01_Accel_Caption, text );
                            
                            DrawImage( &Image_01_03_Accel );
                            DrawLabel( &Label_01_01_Accel );
                        
                            Check_TP( );
                        }
                        
                    #endif

                    #if ( _MMB_MODULE_RTC == 1 )
                    
                        if (status_module.menu == MENU_COLLAPSED)
                        {
                            bsp_rtc_get( &rtc_time );
                        
                            Label_01_04_Rtc_Caption[0] = 0;

                            strcat( Label_01_04_Rtc_Caption, "D: " );
                            text[0] = 0;
                            ByteToStr( rtc_time.d, text );
                            Ltrim( text );
                            strcat( Label_01_04_Rtc_Caption, text );

                            strcat( Label_01_04_Rtc_Caption, " M: " );
                            text[0] = 0;
                            ByteToStr( rtc_time.m, text );
                            Ltrim( text );
                            strcat( Label_01_04_Rtc_Caption, text );
                            
                            strcat( Label_01_04_Rtc_Caption, " Y: " );
                            text[0] = 0;
                            WordToStr( rtc_time.y, text );
                            Ltrim( text );
                            strcat( Label_01_04_Rtc_Caption, text );
                            
                            strcat( Label_01_04_Rtc_Caption, "   |   " );
                            
                            strcat( Label_01_04_Rtc_Caption, "h: " );
                            text[0] = 0;
                            ByteToStr( rtc_time.hr, text );
                            Ltrim( text );
                            strcat( Label_01_04_Rtc_Caption, text );
                            
                            strcat( Label_01_04_Rtc_Caption, " m: " );
                            text[0] = 0;
                            ByteToStr( rtc_time.min, text );
                            Ltrim( text );
                            strcat( Label_01_04_Rtc_Caption, text );
                            
                            strcat( Label_01_04_Rtc_Caption, " s: " );
                            text[0] = 0;
                            ByteToStr( rtc_time.sec, text );
                            Ltrim( text );
                            strcat( Label_01_04_Rtc_Caption, text );

                            DrawImage( &Image_01_06_Rtc );
                            DrawLabel( &Label_01_04_Rtc );

                            Check_TP( );
                        }
                        
                    #endif

                    #if ( _MMB_MODULE_TEMP == 1 )
                    
                        if (( status_module.menu == MENU_COLLAPSED ) && ( aux_ctr == REFRESH_RATE ))
                        {
                            temperature = bsp_temperature_read( );

                            Label_01_03_Temp_Caption[0] = 0;
                           
                            FloatToStr( temperature, Label_01_03_Temp_Caption );
                            Ltrim( Label_01_03_Temp_Caption );
                            float_cut( Label_01_03_Temp_Caption );
                            strcat( Label_01_03_Temp_Caption, "   C" );

                            DrawImage( &Image_01_05_Temp );
                            DrawLabel( &Label_01_03_Temp );
                            
                            Check_TP( );
                        }
                        
                    #endif

                    #if ( _MMB_MODULE_ALS == 1 )
                    
                        if (status_module.menu == MENU_COLLAPSED)
                        {
                            bsp_als_read_measurement( &channel_0, &channel_1 );

                            Label_01_02_Ambient_Caption[0] = 0;

                            text[0] = 0;
                            strcat( Label_01_02_Ambient_Caption, "Ch 0   :   " );
                            WordToStr( channel_0, text );
                            Ltrim( text );
                            strcat( Label_01_02_Ambient_Caption, text );

                            text[0] = 0;
                            strcat( Label_01_02_Ambient_Caption, "   |   Ch 1   :   " );
                            WordToStr( channel_1, text );
                            Ltrim( text );
                            strcat( Label_01_02_Ambient_Caption, text );

                            DrawImage( &Image_01_04_Ambient );
                            DrawLabel( &Label_01_02_Ambient );

                            Check_TP( );
                        }
                        
                    #endif

                    #if ( _MMB_MODULE_WIFI == 1 )
                    
                        if (( status_module.menu == MENU_COLLAPSED ) && ( aux_wifi_ctr == REFRESH_RATE_WIFI ))
                        {
                            retVal = establishConnectionWithAP( );

                            if ( retVal < 0 )
                            {
                                test_console_write_line( "Failed to establish connection w/ an AP" );

                                return;
                            }

                            test_console_write_line( "Connection established w/ AP and IP is acquired" );

                            retVal = getWeather( );

                            if ( retVal < 0 )
                            {
                                test_console_write_line( "Failed to get weather information" );

                                return;
                            }

                            retVal = disconnectFromAP( );
                            
                            if ( retVal < 0 )
                            {
                                test_console_write_line( "Failed to disconnect from AP" );

                                return;
                            }
                            
                            get_weather_station_data( Label_01_03_Temp_Caption, Label_01_07_Humidity_Caption, Label_01_06_Weather_Caption );
                            
                            DrawImage( &Image_01_09_Humidity );
                            DrawLabel( &Label_01_07_Humidity );

                            DrawImage( &Image_01_08_Weather );
                            DrawLabel( &Label_01_06_Weather );

                            Check_TP( );
                        }
                        
                    #endif

                    if (aux_ctr >= REFRESH_RATE) aux_ctr = 0;
                    else aux_ctr++;
                    
                    if (aux_wifi_ctr >= REFRESH_RATE_WIFI) aux_wifi_ctr = 0;
                    else aux_wifi_ctr++;
                    
                    break;
                }
                case MENU_EXPANDED :
                {


                    break;
                }
            }

            break;
        }
        default :
        {
            /** ERROR */

            break;
        }
    }
}

/* --------------------------------------------- PRIVATE FUNCTION DEFINITIONS */

static void float_cut( char * text )
{
    uint8_t count;
    uint8_t conCnt = 0;
    uint8_t conVar = 0;

    for (count = 0; count < 50; count++)
    {
        if (text[ count ] == '.')
        {
            conVar = 1;
        }
        if (conVar == 1)
        {
            conCnt++;
        }
        if (conCnt > 3)
        {
            if ((text[ count ] == 'e') || (conVar == 2))
            {
                text[ count - (conCnt - 4) ] = text[ count ];
                text[ count ] = 0;
                conVar = 2;
            }
            else
            {
                text[ count ] = 0;
            }
        }
    }
}