#include "system_driver.h"
#include "Mikromedia_7_Demo_objects.h"

/** 
    This function enables all interrupts used by application
    See file interrupts.c for further reference
*/
extern void interrupts_enable( void );

/**
    References to all available modules loaded dynamically during 
    initialization.

    Reference index is actually system_module_t enumeration.
*/
static volatile module_t *              system_module[5];

/**
    Currently active module
*/
static volatile system_module_t         active;

/** 
    This function is called by events code
    handlers in order to change menu state
*/
void system_menu_changed ( screen_menu_t new_menu )
{
    system_module[ active ]->menu = new_menu;

    switch ( new_menu ) 
    {
        case MENU_COLLAPSED :
        {
            system_screen_changed( active );

            break;
        }
        case MENU_EXPANDED :
        {
            DrawScreen( &Screen_06_Menu );

            break;
        }
        default :
        {
            /** ERROR */
            
            break;
        }
    }
}

/** 
    This function is called by events code handlers
    in order to change active module
*/
void system_screen_changed ( system_module_t new_module )
{
    /**
        Put all modules to inactive state and set new active module. 

        The reason for this is construction of the module state machine where 
        first loop always have to pass through MODULE_INACTIVE state.
    */
    system_module[ MODULE_STATUS ]->state = MODULE_INACTIVE;
    system_module[ MODULE_CONFIG ]->state = MODULE_INACTIVE;
    system_module[ MODULE_MP3    ]->state = MODULE_INACTIVE;
    system_module[ MODULE_REMOTE ]->state = MODULE_INACTIVE;
    system_module[ MODULE_INFO   ]->state = MODULE_INACTIVE;

    active = new_module;
    
    /**
        Set menu state to collapsed state
    */
    system_module[ active ]->menu = MENU_COLLAPSED;
}

void system_init ( void )
{
    /**
        Enable interrupts
    */
    interrupts_enable( );

    /**
        Initialize system modules
    */
    status_module_init( &system_module[ MODULE_STATUS ] );
    config_module_init( &system_module[ MODULE_CONFIG ] );
    mp3_module_init( &system_module[ MODULE_MP3 ] );
    remote_module_init( &system_module[ MODULE_REMOTE ] );
    info_module_init( &system_module[ MODULE_INFO ] );
    
    /**
        Set first screen
    */
    system_screen_changed( MODULE_STATUS );
}

void system_process ( void )
{
    switch ( active )
    {
        case MODULE_STATUS :
        {
            status_module_task( );
            break;
        }
        case MODULE_CONFIG :
        {
            config_module_task( );
            break;
        }
        case MODULE_MP3 :
        {
            mp3_module_task( );
            break;
        }
        case MODULE_REMOTE :
        {
            remote_module_task( );
            break;
        }
        case MODULE_INFO :
        {
            info_module_task( );
            break;
        }
        default :
        {
            /** ERROR */
            break;
        }
    }
}