/*
    bsp_rtc_driver.c

-----------------------------------------------------------------------------

  This file is part of mikroSDK.

  Copyright (c) 2017, MikroElektonika - http://www.mikroe.com

  All rights reserved.

----------------------------------------------------------------------------- */

#include "hal.h"
#include "bsp_rtc_driver.h"

#if ( _MMB_MODULE_RTC == 1 )

/* ---------------------------------------------------------------- VARIABLES */

unsigned long RTC_Time, old_RTC_Time;
unsigned long RTC_Date, old_RTC_Date;

/* ----------------------------------------------------------- IMPLEMENTATION */

void bsp_rtc_init ( void )
{
    hal_rtc_init( );
}

void bsp_rtc_get ( rtc_time_t * time )
{
    char temp = 0;

    RTC_Time = RTC_TR;
    RTC_Date = RTC_DR;

    if(RTC_Time != old_RTC_Time){
      old_RTC_Time = RTC_Time;

      time->ampm = (short)((RTC_Time & 0x400000) >> 22);
      time->hr = Bcd2Dec((short)((RTC_Time & 0x3f0000) >> 16));
      time->min = Bcd2Dec((short)((RTC_Time & 0x007f00) >> 8));
      time->sec = Bcd2Dec((short)(RTC_Time & 0x0000ff));

      temp = 1;
    }

    if(RTC_Date != old_RTC_Date){
      old_RTC_Date = RTC_Date;

      time->y = Bcd2Dec((short)((RTC_Date & 0x3f0000) >> 16));
      time->m = Bcd2Dec((short)((RTC_Date & 0x001f00) >> 8));
      time->d = Bcd2Dec((short)(RTC_Date & 0x0000ff));

      temp = 1;
    }
}

void bsp_rtc_set ( rtc_time_t * time )
{
    unsigned long temp;

    PWR_CR1.DBP = 1;

    RTC_WPR = 0xCA;                   //unlock write protection
    RTC_WPR = 0x53;                   //unlock write protection
    RTC_ISR = 0x00000080;             //enter initialization mode  bit 7
    while(RTC_ISR.INITF != 1){};      // bit 6

    RTC_PRER = 0x7f00ff;              //  set   SynchPrediv to FF and AsynchPrediv to 7F*/

    temp = (unsigned long)Dec2Bcd(time->hr) << 16;
    temp += (unsigned long)Dec2Bcd(time->min) << 8;
    temp += (unsigned long)Dec2Bcd(time->sec);
    temp += (unsigned long)(time->ampm) << 22;

    RTC_TR = temp;

    temp = (unsigned long)Dec2Bcd(time->d) << 16;
    temp += (unsigned long)Dec2Bcd(time->m) << 8;
    temp += (unsigned long)Dec2Bcd(time->y);

    RTC_DR = temp;           // set date
    RTC_CRbits.FMT = 1;
    RTC_CRbits.WCKSEL = 0;   // set FMT 12H format

    RTC_ISR = 0x00000000;    //exit initialization mode

    RTC_WPR = 0xFF;          // Enable the write protection for RTC registers
}

#endif

/* -------------------------------------------------------------------------- */
/*
    bsp_rtc_driver.c

  Copyright (c) 2017, MikroElektronika - http://www.mikroe.com

  All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. All advertising materials mentioning features or use of this software
   must display the following acknowledgment:
   This product includes software developed by the MikroElektronika.

4. Neither the name of the MikroElektronika nor the
   names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY MIKROELEKTRONIKA ''AS IS'' AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL MIKROELEKTRONIKA BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

----------------------------------------------------------------------------- */