/*
    api_wifi_handlers.c

-----------------------------------------------------------------------------

  This file is part of mikroSDK.

  Copyright (c) 2017, MikroElektonika - http://www.mikroe.com

  All rights reserved.

----------------------------------------------------------------------------- */

#include "sl_common.h"
#include "test_console.h"
#include "api_wifi_handlers.h"

#if ( _MMB_MODULE_WIFI == 1 )

/* ---------------------------------------------------------------- VARIABLES */

extern _u32 g_Status;

/* ----------------------------------------------------------- IMPLEMENTATION */

void api_wifi_prerror ( int error_code )
{ 
    char txt[32];
    
    if ( error_code < 0 )
    {
        IntToStr( error_code, txt );
        Ltrim( txt );

        test_console_write_text( "ERR :" );
        test_console_write_line( txt );
    }
}

void api_wifi_general_event_handler ( SlDeviceEvent_t * p_device_event )
{
    test_console_write_text( "[WIFI GENERAL EVENT] " );
    test_console_write_line( "- Unexpected event");
}

void api_wifi_wlan_event_handler ( SlWlanEvent_t * p_wlan_event )
{
    test_console_write_text( "[WIFI WLAN EVENT] ");

    if ( p_wlan_event == NULL )
    {
        test_console_write_line( "- NULL Pointer Error" );
        return;
    }

    switch ( p_wlan_event->Event )
    {
        case SL_WLAN_CONNECT_EVENT:
        {
            slWlanConnectAsyncResponse_t * pEventData = 
                    ( slWlanConnectAsyncResponse_t * ) &p_wlan_event->EventData;

            test_console_write_text( "- Connected to : " );
            test_console_write_line( pEventData->ssid_name );

            SET_STATUS_BIT( g_Status, STATUS_BIT_CONNECTION );

            break;
        }
        case SL_WLAN_DISCONNECT_EVENT:
        {
            slWlanConnectAsyncResponse_t*  pEventData = NULL;

            CLR_STATUS_BIT( g_Status, STATUS_BIT_CONNECTION );
            CLR_STATUS_BIT( g_Status, STATUS_BIT_IP_ACQUIRED );

            pEventData = &p_wlan_event->EventData.STAandP2PModeDisconnected;

            if ( SL_USER_INITIATED_DISCONNECTION == pEventData->reason_code )
            {
                test_console_write_line( "- Device disconnected from the AP on request" );
            }
            else
            {
                test_console_write_line( "- Device disconnected from the AP on an ERROR" );
            }

            break;
        }
        default:
        {
            test_console_write_line( "- Unexpected event" );

            break;
        }
    }
}

void api_wifi_net_app_event_handler ( SlNetAppEvent_t * p_net_app_event )
{
    test_console_write_text( "[WIFI NETAPP EVENT] " );

    if ( p_net_app_event == NULL )
    {
        test_console_write_line( "- NULL Pointer Error" );
        return;
    }

    switch ( p_net_app_event->Event )
    {
        case SL_NETAPP_IPV4_IPACQUIRED_EVENT:
        {
            SET_STATUS_BIT( g_Status, STATUS_BIT_IP_ACQUIRED );

            break;
        }
        default:
        {
            test_console_write_line( "- Unexpected event" );

            break;
        }
    }
}

void api_wifi_socket_event_handler ( SlSockEvent_t * p_socket )
{
    test_console_write_text( "[WIFI SOCKET EVENT] " );

    if ( p_socket == NULL )
    {
        test_console_write_line( "- NULL Pointer Error" );

        return;
    }

    switch ( p_socket->Event )
    {
        case SL_SOCKET_TX_FAILED_EVENT:
        {
            switch ( p_socket->socketAsyncEvent.SockTxFailData.status )
            {
                case SL_ECLOSE:
                {
                    test_console_write_line( "- Close socket operation failed to transmit all queued packets" );

                    break;
                }
                default:
                {
                    test_console_write_line( "- Unexpected event");

                    break;
                }
            }

            break;
        }
        default:
        {
            test_console_write_line( "- Unexpected event");

            break;
        }
    }
}

void api_wifi_http_server_callback ( SlHttpServerEvent_t * p_http_event, SlHttpServerResponse_t * p_http_response )
{
    test_console_write_text( "[WIFI HTTP EVENT] ");
    test_console_write_line( "- Unexpected event");
}

#endif

/* -------------------------------------------------------------------------- */
/*
    api_wifi_handlers.c

  Copyright (c) 2017, MikroElektronika - http://www.mikroe.com

  All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. All advertising materials mentioning features or use of this software
   must display the following acknowledgment:
   This product includes software developed by the MikroElektronika.

4. Neither the name of the MikroElektronika nor the
   names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY MIKROELEKTRONIKA ''AS IS'' AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL MIKROELEKTRONIKA BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

----------------------------------------------------------------------------- */